#include <windows.h>


#define rGPEDAT  (*(volatile unsigned *)0xB1600044)
#define rGPECON1  (*(volatile unsigned *)0xB1600040)
#define  rGPEUP1   (*(volatile unsigned *)0xB1600048)

#define WAIT_STAB udelay(100)   //msWait(1)
#define WAIT_CYL  udelay(100)

#define OV7620_SCCB_ID 0x60  /* CS[2:0] = 000 */	//hzh, use variable

/************************************************/
//some macros and inline function to support porting
//from linux driver, hzh
//#define	CLKCON					rCLKCON
#define	CLKCON_CAMIF		(1<<19)
//#define	UPLLCON					rUPLLCON
//#define	CLKDIVN					rCLKDIVN
#define	DIVN_UPLL_EN			(1<<3)
//#define	CAMDIVN					rCAMDIVN
#define	CAMCLK_SET_DIV			(1<<4)


#define	GPIO_E14				14
#define	GPIO_E15				15
#define	GPIO_PULLUP_EN			(0<<30)
#define	GPIO_PULLUP_DIS			(1<<30)
#define	GPIO_MODE_IN			(0<<31)
#define	GPIO_MODE_OUT			(1UL<<31)

#define	read_gpio_bit(x)		((rGPEDAT & (1<<(x))) ? 1 : 0)
#define	write_gpio_bit_set(x)	(rGPEDAT |= 1<<(x))
#define	write_gpio_bit_clear(x)	(rGPEDAT &= ~(1<<(x)))


#define SET_SIOD_READ()     rGPECON1=(rGPECON1&0x3FFFFFFF)
#define SET_SIOD_WRITE()    rGPECON1=(rGPECON1&0x3FFFFFFF)|0x40000000
#define SET_SIOC_READ()     rGPECON1=(rGPECON1&0xCFFFFFFF)
#define SET_SIOC_WRITE()    rGPECON1=(rGPECON1&0xCFFFFFFF)|0x10000000


#define	udelay(x)		Delay(x/50)
#define	mdelay(x)		Delay((x)*8)


extern void Delay(USHORT count);

static __inline void set_gpio_ctrl(unsigned int gpio)
{
	rGPECON1 &= ~(3<<((gpio&0xf)*2));
	if(gpio&GPIO_MODE_OUT)
		rGPECON1 |= 1<<((gpio&0xf)*2);
	if(gpio&GPIO_PULLUP_DIS)
		rGPEUP1 |= 1<<(gpio&0xf);
	else
		rGPEUP1 &= ~(1<<(gpio&0xf));
}


#define SIO_C          (GPIO_E14)
#define SIO_D          (GPIO_E15)
#define MAKE_HIGH(_x)  write_gpio_bit_set(_x)
#define MAKE_LOW(_x)   write_gpio_bit_clear(_x)
#define BIT_READ(_x)   read_gpio_bit(_x)
#define CFG_READ(_x)   set_gpio_ctrl(_x | GPIO_PULLUP_DIS | GPIO_MODE_IN)
#define CFG_WRITE(_x)  set_gpio_ctrl(_x | GPIO_PULLUP_DIS | GPIO_MODE_OUT)


/* 2-wire SCCB */
void __inline ov7620_sccb_start(void)
{
	MAKE_HIGH(SIO_C);
	MAKE_HIGH(SIO_D);
	WAIT_STAB;
	MAKE_LOW(SIO_D);
	WAIT_STAB;
	MAKE_LOW(SIO_C);
	WAIT_STAB;
}

/* 2-wire SCCB */
void __inline ov7620_sccb_end(void)
{
	MAKE_LOW(SIO_D);
	WAIT_STAB;
	MAKE_HIGH(SIO_C);
	WAIT_STAB;
	MAKE_HIGH(SIO_D);
	WAIT_STAB;
}

void __inline ov7620_sccb_write_bit(unsigned char bit)
{
	if (bit)
		MAKE_HIGH(SIO_D);
	else
		MAKE_LOW(SIO_D);
	WAIT_STAB;
	MAKE_HIGH(SIO_C);
	WAIT_CYL;
	MAKE_LOW(SIO_C);
	WAIT_STAB;
}

int __inline ov7620_sccb_read_bit(void)
{
	int tmp = 0;

	MAKE_HIGH(SIO_C);
	WAIT_CYL;
	tmp = BIT_READ(SIO_D);
	MAKE_LOW(SIO_C);
	WAIT_STAB;

	return tmp;
}	

void __inline ov7620_sccb_writechar(unsigned char data)
{
	int i = 0;

	/* data */
	for (i = 0; i < 8; i++ ) {
		ov7620_sccb_write_bit(data & 0x80);
		data <<= 1;
	}

	/* 9th bit - Don't care */
	ov7620_sccb_write_bit(1);
}

void __inline ov7620_sccb_readchar(unsigned char *val)
{
	int i;
	int tmp = 0;

	CFG_READ(SIO_D);

	for (i = 7; i >= 0; i--)
		tmp |= ov7620_sccb_read_bit() << i;

	CFG_WRITE(SIO_D);

	/* 9th bit - N.A. */
	ov7620_sccb_write_bit(1);

	*val = tmp & 0xff;
}

/* 3-phase write */
void ov7620_sccb_sendbyte(unsigned char subaddr, unsigned char data)
{
//	down(&dev.bus_lock);

	ov7620_sccb_start();
	ov7620_sccb_writechar(OV7620_SCCB_ID);
	ov7620_sccb_writechar(subaddr);
	ov7620_sccb_writechar(data);
	ov7620_sccb_end();

	mdelay(1);

//	up(&dev.bus_lock);
}

/* 2-phase read */
unsigned char ov7620_sccb_receivebyte(unsigned char subaddr)
{
	unsigned char value;

//	down(&dev.bus_lock);

	/* 2-phase write */
	ov7620_sccb_start();
	ov7620_sccb_writechar(OV7620_SCCB_ID);
	ov7620_sccb_writechar(subaddr);
	ov7620_sccb_end();

	/* 2-phase read */
	ov7620_sccb_start();
	ov7620_sccb_writechar(OV7620_SCCB_ID | 0x01);
	ov7620_sccb_readchar(&value);
	ov7620_sccb_end();

	//msWait(7);
	mdelay(7);

//	up(&dev.bus_lock);

	return value;
}


void ReadCameraProductID(void)
{
    BYTE id ;
	id = 0 ;

	CFG_WRITE(SIO_C);
	CFG_WRITE(SIO_D);
	mdelay(10);
	
	id = ov7620_sccb_receivebyte(10);
	
	RETAILMSG(1,(TEXT("ID=0x%x\r\n"),id));
    
    //Rd_CamIIC(SlaveID,10,&id) ;
	
  //  RETAILMSG(1,(TEXT("------------------\r\n")));
   //RETAILMSG(1,(TEXT("Product ID = 0x%x\r\n"), id));
	
}