//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
//------------------------------------------------------------------------------
//
//  File:  bsp_cfg.h
//
//  This file contains system constant specific for SMDK2440A board.
//
#ifndef __BSP_CFG_H
#define __BSP_CFG_H


//------------------------------------------------------------------------------
// Misc defines.
//------------------------------------------------------------------------------
#define D1_2			0x0
#define D1_4			0x1
#define D1_8			0x2
#define D1_16			0x3
#define D2				2
#define D4				4
#define D8				8
#define D16				16

//------------------------------------------------------------------------------
//
//  Define:  BSP_DEVICE_PREFIX
//
//  Prefix used to generate device name for bootload/KITL
//
#define BSP_DEVICE_PREFIX       "SMDK2440"        // Device name prefix
//------------------------------------------------------------------------------
// Board clock
//------------------------------------------------------------------------------

#define S3C2440A_BASE_REG_VA_CLOCK_POWER	0xB0C00000
#define PLLVALUE			(((S3C2440A_CLKPWR_REG*)(S3C2440A_BASE_REG_VA_CLOCK_POWER))->MPLLCON)
#define CLKDIV				(((S3C2440A_CLKPWR_REG*)(S3C2440A_BASE_REG_VA_CLOCK_POWER))->CLKDIVN)
#define CAMDIV				(((S3C2440A_CLKPWR_REG*)(S3C2440A_BASE_REG_VA_CLOCK_POWER))->CAMDIVN)

//#define S3C2440A_FCLK		(2 * (((PLLVALUE >> 12) & 0xff) + 8L) * (16934400L / (((PLLVALUE >> 4) & 0x3f)+2L) / (1<<((PLLVALUE >> 0) & 0x3))))	
// PLLVALUE = v_pCLKPWRreg->MPLLCON
#define S3C2440A_FCLK		(2 * (((PLLVALUE >> 12) & 0xff) + 8L) * (12000000L / (((PLLVALUE >> 4) & 0x3f)+2L) / (1<<((PLLVALUE >> 0) & 0x3))))
// PLLVALUE = v_pCLKPWRreg->MPLLCON

#define HDIVN				((CLKDIV>>1) & 0x3)
#define	PDIVN				((CLKDIV>>0) & 0x1)
#define HCLK3_HALF			((CAMDIV >> 8) & 0x1)
#define HCLK4_HALF			((CAMDIV >> 9) & 0x1)
#define HCLKDIV				((HDIVN & 0x2) == 0 ?														\
									(HDIVN == 0 ? 1:2) :											\
									(HDIVN == 2 ?												\
											(HCLK4_HALF == 0 ? 4:8) :								\
											(HCLK3_HALF == 0 ? 3:6)))										// CLKDIV = v_pCLKPWRreg->CLKDIVN
#define	PCLKDIV				(HCLKDIV * ((PDIVN == 0) ? 1 : 2))				// CLKDIV = v_pCLKPWRreg->CLKDIVN

#define S3C2440A_HCLK		(S3C2440A_FCLK/HCLKDIV)   
#define S3C2440A_PCLK		(S3C2440A_FCLK/PCLKDIV)   
#define SYS_TIMER_DIVIDER	D2
#define PRESCALER			((S3C2440A_PCLK/(1000000*SYS_TIMER_DIVIDER)) - 1)

#define OEM_CLOCK_FREQ		(S3C2440A_PCLK/(PRESCALER+1)/SYS_TIMER_DIVIDER)

// Timer count for 1 ms
#define OEM_COUNT_1MS       (OEM_CLOCK_FREQ / 1000)	// OEM_CLOCK_FREQ = 1000000, 1000000/1000 => 1000 = 1 msec
#define RESCHED_PERIOD      1		// 10				// Reschedule period in ms

// Timer count value for rescheduler period (???? for RESCHED_PERIOD ms system tick)
#define RESCHED_INCREMENT   ( RESCHED_PERIOD * OEM_COUNT_1MS )

// -----------------------------------------------------------------------------
// define For DVS
#define V080	0
#define V090	1
#define V095	2
#define V0975	3
#define V100	4
#define V105	5
#define V110	6
#define V115	7
#define V120	8
#define V125	9
#define V130	10
#define V135	11
#define V140	12
#define V145	13
#define V150	14

#define DVS_METHOD	3  //1:DVS_ON with VSYNC, 2:DVS_ON in idle mode(not change HCLK), 3:mixed
#define USESWPWSAVING	1
#define MVAL_USED		0
#define HIGHVOLTAGE		V130
#define MIDVOLTAGE		V110
#define LOWVOLTAGE		V100
#define VOLTAGEDELAY	16000

#define Eval_Probe		1

#define DVSON		0x1
#define HCLKHALF	0x2
#define ACTIVE		0x4
#define DeepIdle	(DVSON|HCLKHALF)
#define NIdle		(DVSON)
#define LazyActive	(ACTIVE|DVSON|HCLKHALF)
#define SlowActive	(ACTIVE|DVSON)
#define Active		(ACTIVE)
//------------------------------------------------------------

//------------------------------------------------------------------------------
// Debug UART1
//------------------------------------------------------------------------------

#define DEBUG_BAUDRATE                  (115200)

#define BSP_UART1_ULCON         0x03                // 8 bits, 1 stop, no parity
#define BSP_UART1_UCON          0x0005              // pool mode, PCLK for UART
#define BSP_UART1_UFCON         0x00                // disable FIFO
#define BSP_UART1_UMCON         0x00                // disable auto flow control
#define BSP_UART1_UBRDIV        (S3C2440A_PCLK/(DEBUG_BAUDRATE*16) - 1)

//------------------------------------------------------------------------------
// Static SYSINTR Mapping for driver.
#define SYSINTR_OHCI            (SYSINTR_FIRMWARE+1)
#endif
